"""Tests for update_project.py"""
from pathlib import Path

import pytest

from nens_meta import nens_toml, update_project


def test_check_prerequisites1(tmp_path: Path):
    with pytest.raises(SystemExit):
        # Missing .git/
        update_project.check_prerequisites(tmp_path)


def test_check_prerequisites2(tmp_path: Path):
    (tmp_path / ".git").mkdir()
    with pytest.raises(SystemExit):
        # Missing .nens.toml
        update_project.check_prerequisites(tmp_path)


# TemplatedFile is tested through EditorConfig, btw
def test_editor_config1(tmp_path: Path):
    # No config, check file contents.
    nens_toml.create_if_missing(tmp_path)
    our_config = nens_toml.OurConfig(tmp_path)
    editor_config = update_project.Editorconfig(tmp_path, our_config)
    assert "geojson" in editor_config.content


def test_editor_config2(tmp_path: Path):
    # No config, check file contents of the written file
    nens_toml.create_if_missing(tmp_path)
    our_config = nens_toml.OurConfig(tmp_path)
    editor_config = update_project.Editorconfig(tmp_path, our_config)
    editor_config.write()
    assert "geojson" in (tmp_path / ".editorconfig").read_text()


def test_dependabot(tmp_path: Path):
    nens_toml.create_if_missing(tmp_path)
    our_config = nens_toml.OurConfig(tmp_path)
    dependabot_yml = update_project.DependabotYml(tmp_path, our_config)
    assert "interval" in dependabot_yml.content
    dependabot_yml.write()


def test_metaworkflowyml(tmp_path: Path):
    nens_toml.create_if_missing(tmp_path)
    our_config = nens_toml.OurConfig(tmp_path)
    meta_workflow_yml = update_project.MetaWorkflowYml(tmp_path, our_config)
    assert "workflow_dispatch" in meta_workflow_yml.content
    meta_workflow_yml.write()


def test_requirements_yml1(tmp_path):
    # requirements.yml should be created if it doesn't exist yet.
    nens_toml.create_if_missing(tmp_path)
    our_config = nens_toml.OurConfig(tmp_path)
    requirements_yml = update_project.RequirementsYml(tmp_path, our_config)
    requirements_yml.write()
    content = (tmp_path / "requirements.yml").read_text()
    assert "ansible.posix" in content


def test_requirements_yml2(tmp_path: Path):
    # requirements.yml should only be created if it doesn't exist yet.
    nens_toml.create_if_missing(tmp_path)
    existing_file = tmp_path / "requirements.yml"
    existing_file.write_text("Hurray, some existing content")
    our_config = nens_toml.OurConfig(tmp_path)
    requirements_yml = update_project.RequirementsYml(tmp_path, our_config)
    requirements_yml.write()
    content = (tmp_path / "requirements.yml").read_text()
    assert "ansible.posix" not in content
    assert "Hurray" in content


def test_requirements_yml(tmp_path):
    # requirements.yml should be only be created if it doesn't exist yet, so there
    # should not be the "Extra lines below are preserved" instruction in there.
    nens_toml.create_if_missing(tmp_path)
    our_config = nens_toml.OurConfig(tmp_path)
    requirements_yml = update_project.RequirementsYml(tmp_path, our_config)
    requirements_yml.write()
    content = (tmp_path / "requirements.yml").read_text()
    assert "Extra lines below" not in content



"""Tests for update_project.py"""
from pathlib import Path

import pytest

from nens_meta import nens_toml


def test_nens_toml_file(tmp_path: Path):
    assert nens_toml.nens_toml_file(tmp_path).name == ".nens.toml"


def test_create_if_missing(tmp_path: Path):
    nens_toml.create_if_missing(tmp_path)
    assert (tmp_path / ".nens.toml").exists()


def test_init(tmp_path: Path):
    nens_toml.nens_toml_file(tmp_path).write_text("year = 1972")
    config = nens_toml.OurConfig(tmp_path)
    assert config._contents["year"] == 1972


def test_write(tmp_path: Path):
    nens_toml.nens_toml_file(tmp_path).write_text("year = 1972")
    config = nens_toml.OurConfig(tmp_path)
    config._contents["month"] = 12
    config.write()
    assert "month = 12" in nens_toml.nens_toml_file(tmp_path).read_text()


def test_section_options1(tmp_path: Path):
    # Properly read a known variable in a known section.
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [meta]
    project_name = "1972"
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    assert config.section_options("meta")["project_name"] == "1972"


def test_section_options2(tmp_path: Path):
    # Barf upon an unknown/undocumented section.
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [reinout]
    year = 1972
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    with pytest.raises(nens_toml.MissingDocumentationError):
        config.section_options("reinout")


def test_section_options3(tmp_path: Path):
    # Don't return values that are unknown.
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [meta]
    year = 1972
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    assert "year" not in config.section_options("meta").keys()


def test_section_options_boolean1(tmp_path: Path):
    # Handle boolean values (those have _TRUE or _FALSE prepended)
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [meta]
    uses_python = true
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    assert config.section_options("meta")["uses_python"] is True


def test_section_options_boolean2(tmp_path: Path):
    # Handle default for boolean values
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [meta]
    # uses_python has a default of false
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    assert config.section_options("meta")["uses_python"] is False


def test_section_options_boolean3(tmp_path: Path):
    # Complain if a boolean value has a non-boolean value.
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [meta]
    uses_python = "1972"
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    with pytest.raises(ValueError):
        config.section_options("meta")["uses_python"]


def test_update_meta_options1(tmp_path: Path):
    # Create a new "meta" section if it is missing.
    nens_toml.nens_toml_file(tmp_path).write_text("")
    config = nens_toml.OurConfig(tmp_path)
    # the init automatically calls config.update_meta_options()
    assert "meta" in config._contents


def test_update_meta_options2(tmp_path: Path):
    # version and so is filled in in an updated file.
    nens_toml.nens_toml_file(tmp_path).write_text("")
    config = nens_toml.OurConfig(tmp_path)
    # the init automatically calls config.update_meta_options()
    meta_section = config.section_options("meta")
    assert "meta_version" in meta_section
    assert "project_name" in meta_section


def test_update_meta_options4(tmp_path: Path):
    # The version should always be updated.
    nens_toml.nens_toml_file(tmp_path).write_text(
        """
    [meta]
    meta_version = "1972"
    """
    )
    config = nens_toml.OurConfig(tmp_path)
    assert config.section_options("meta")["meta_version"] != "1972"


def test_write_documentation():
    nens_toml.write_documentation()



import logging
import re
from pathlib import Path

logger = logging.getLogger(__name__)

EXTRA_LINES_MARKER = "### Extra lines below are preserved ###\n"
LEAVE_ALONE_MARKER = "NENS_META_LEAVE_ALONE"
SUGGESTION_SUFFIX = ".suggestion"


def strip_whitespace(content: str) -> str:
    """Return content stripped of EOL whitespace and extra EOF linefeeds

    Generating files using jinja2 template tags sometimes result in such unneeded
    whitespace. Stripping it off afterwards is easier than trying to get it right with
    jinja2.
    """
    # Get rid of spaces at the end of lines.
    content = re.sub(r"\ +\n", r"\n", content)
    # Get rid of empty lines at the end.
    content = content.strip() + "\n"
    return content


def _extract_extra_lines(content: str) -> str:
    """Return content after the extra lines marker"""
    parts = content.split(EXTRA_LINES_MARKER)
    if len(parts) > 1:
        return parts[1]
    else:
        return ""


def write_if_changed(target: Path, desired_content: str, handle_extra_lines=True):
    """Write content to file if different, not if it is the same

    And create the file if it doesn't exist.

    And... look for an end-of-generated-file marker and preserve the contents after
    it. If `handle_extra_lines` is True (the default).

    And... leave it alone if the marker is there.

    """
    existing_content = target.read_text() if target.exists() else ""
    leave_alone = LEAVE_ALONE_MARKER in existing_content
    if handle_extra_lines:
        extra_lines = _extract_extra_lines(existing_content)
        extra_lines_marker_with_empty_line_before = "\n" + EXTRA_LINES_MARKER
        new_content = extra_lines_marker_with_empty_line_before.join(
            [desired_content, extra_lines]
        )
    else:
        new_content = desired_content

    if new_content == existing_content:
        logger.debug(f"{target} remained the same")
        return

    if leave_alone:
        logger.debug(f"Leave-alone marger found in {target}")
        target = target.parent / (target.name + SUGGESTION_SUFFIX)

    target.write_text(new_content)
    logger.info(f"Wrote {target}")


def uses_python(project: Path) -> bool:
    """Return whether we detect a python project"""
    if any(project.glob("**/*.py")):
        logger.debug("*.py found, assuming we use python")
        return True
    return False


def uses_ansible(project: Path) -> bool:
    """Return whether we detect an ansible dir"""
    if (project / "ansible").exists():
        logger.debug("ansible/ dir found, assuming we use ansible")
        return True
    return False



"""Purpose: read and manage the .nens.toml config file
"""
import copy
import logging
from dataclasses import dataclass
from pathlib import Path
from typing import Any

import tomlkit
from tomlkit.items import Table

from nens_meta import __version__, utils


@dataclass
class Option:
    key: str
    description: str
    value_type: type = str
    default: Any = ""


META_FILENAME = ".nens.toml"
KNOWN_SECTIONS: dict[str, list[Option]] = {}
# First key is the section name, the second key/value pair is the variable name and the
# explanation. If the second key ends with "_TRUE"/"_FALSE", this is stripped and will
# be used to treat the value as a boolean with the indicated default.
KNOWN_SECTIONS["meta"] = [
    Option(key="meta_version", description="Version used to generate the config"),
    Option(
        key="project_name",
        description="Project name (normally the name of the directory)",
    ),
    Option(
        key="uses_python",
        description="Whether we use python",
        default=False,
        value_type=bool,
    ),
    Option(
        key="uses_ansible",
        description="Whether we have an ansible dir",
        default=False,
        value_type=bool,
    ),
]
KNOWN_SECTIONS["pyprojecttoml"] = []
KNOWN_SECTIONS["meta_workflow"] = [
    Option(
        key="main_python_version",
        description="Python version to use for linting and so",
        default="3.12",
    ),
]

logger = logging.getLogger(__name__)


def write_documentation():
    target = Path(__file__).parent.parent / "doc" / "nens_toml_example.toml"
    lines = []
    for section in KNOWN_SECTIONS:
        lines.append(f"[{section}]")
        for option in KNOWN_SECTIONS[section]:
            lines.append(f"# {option.description}")
            lines.append(f"{option.key} = {repr(option.default)}")
        lines.append("")

    content = "\n".join(lines)
    content = content.replace(" False", " false")  # Toml syntax hack
    target.write_text(content)


def nens_toml_file(project: Path) -> Path:
    return project / META_FILENAME


def create_if_missing(project: Path):
    if not nens_toml_file(project).exists():
        nens_toml_file(project).write_text("")
    our_config = OurConfig(project)
    our_config.read()
    our_config.update_meta_options()
    our_config.write()


def detected_meta_values(project: Path) -> dict[str, str | bool | list]:
    """Return values we can detect about the project, normally set in [meta]"""
    detected: dict[str, str | bool | list] = {}
    detected["uses_python"] = utils.uses_python(project)
    detected["uses_ansible"] = utils.uses_ansible(project)
    detected["meta_version"] = __version__
    name = project.resolve().name
    detected["project_name"] = name
    return detected


class MissingDocumentationError(Exception):
    pass


class OurConfig:
    """Wrapper around a project's .nens.toml

    See https://tomlkit.readthedocs.io/en/latest/quickstart/
    """

    _config_file: Path
    _contents: tomlkit.TOMLDocument
    _project: Path

    def __init__(self, project: Path):
        self._project = project
        self._config_file = nens_toml_file(project)
        self._contents = self.read()
        self.update_meta_options()

    def read(self) -> tomlkit.TOMLDocument:
        return tomlkit.parse(self._config_file.read_text())

    def write(self):
        utils.write_if_changed(
            self._config_file, tomlkit.dumps(self._contents), handle_extra_lines=False
        )

    def update_meta_options(self):
        """Detect meta options"""
        if "meta" not in self._contents:
            self._contents.append("meta", tomlkit.table())
        current: Table = self._contents["meta"]  # type: ignore
        detected = detected_meta_values(self._project)
        must_be_set = ["meta_version"]
        for key, value in detected.items():
            if key not in current:
                current[key] = value
                logger.info(f".nens.toml: suggesting [meta]->{key}")
            if key in must_be_set:
                if current["meta_version"] != detected["meta_version"]:
                    current["meta_version"] = detected["meta_version"]
                    logger.info(".nens.toml: changing [meta]->meta_version")

    def has_section_for(self, section_name: str) -> bool:
        return section_name in KNOWN_SECTIONS

    def section_options(self, section_name: str) -> dict:
        """Return all options configured in a given section, if available."""
        if section_name not in KNOWN_SECTIONS:
            # Force ourselves to document our stuff!
            raise MissingDocumentationError(
                f"Section {section_name} not documented in nens-meta"
            )
        section = self._contents.get(section_name)
        if section is None:
            section = {}
        options: dict[str, str | bool | list] = {}
        for option in KNOWN_SECTIONS[section_name]:
            value = section.get(option.key, copy.deepcopy(option.default))
            if not isinstance(value, option.value_type):
                raise ValueError(
                    f"{option.key} should be of type {option.value_type}, not {type(value)}"
                )
            options[option.key] = value
        logger.debug(f"Contents of section {section_name}: {options}")
        return options


if __name__ == "__main__":
    # Only called to write the documentation file.
    write_documentation()



"""Purpose: read and manage the pyproject.toml config file
"""
import logging
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any

import tomlkit
from tomlkit.items import Table
from tomlkit.toml_document import TOMLDocument

from nens_meta import utils

FILENAME = "pyproject.toml"

logger = logging.getLogger(__name__)


def pyproject_toml_file(project: Path) -> Path:
    return project / FILENAME


def create_if_missing(project: Path):
    if not pyproject_toml_file(project).exists():
        pyproject_toml_file(project).write_text("# Initially generated by nens-meta\n")
        logger.info("Created empty pyproject.toml")


def write_documentation():
    options = {"project_name": "example-project"}
    target = Path(__file__).parent.parent / "doc" / "pyproject_toml_example.toml"
    with TemporaryDirectory() as project_dir:
        project_dir = Path(project_dir)
        package_dir = project_dir / "example_project"
        package_dir.mkdir()
        create_if_missing(project_dir)
        project_config = PyprojectToml(project_dir, options)
        project_config.update()
        project_config.write()
        project_config._config_file.replace(target)


class PyprojectToml:
    """Wrapper around a project's pyproject.toml"""

    _project: Path
    _config_file: Path
    _contents: tomlkit.TOMLDocument
    _options: dict

    def __init__(self, project: Path, options: dict):
        self._project = project
        self._config_file = pyproject_toml_file(project)
        self._options = options
        self._contents = self.read()

    def read(self) -> tomlkit.TOMLDocument:
        return tomlkit.parse(self._config_file.read_text())

    def write(self):
        target = self._project / FILENAME
        utils.write_if_changed(
            target, tomlkit.dumps(self._contents), handle_extra_lines=False
        )

    def get_or_create_section(self, name: str) -> Table:
        *super_tables, section_name = name.split(".")
        current_container: TOMLDocument | Table = self._contents
        for super_table in super_tables:
            if super_table not in current_container:  # type: ignore
                current_container.append(
                    super_table, tomlkit.table(is_super_table=True)
                )  # type: ignore
                logger.debug(f"Created section parent {super_table} for {name}")
            current_container = current_container[super_table]  # type: ignore

        if section_name not in current_container:
            current_container.append(section_name, tomlkit.table())
            logger.debug(f"Created section {name}")
        section: Table = current_container[section_name]  # type: ignore
        return section

    def update(self):  # pragma: no cover
        """Update the pyproject.toml file

        `options` is the combined contents of the [meta] and [pyprojecttoml] config
        sections.
        """

        self.adjust_ruff()
        self.adjust_zestreleaser()
        self.remove_old_sections()

    def _suggest(self, section_name: str, key: str, value: Any, strongly=False):
        section = self.get_or_create_section(section_name)
        if key not in section:
            section[key] = value
            logger.info(f"pyproject.toml: suggesting [{section_name}]->{key}")
        if strongly:
            if section[key] != value:
                logger.info(
                    f"    Note: our suggested pyproject.toml value for [{section_name}]->{key}: {value}"
                )

    def _force(self, section_name: str, key: str, value: Any):
        section = self.get_or_create_section(section_name)
        if section.get(key) != value:
            section[key] = value
            logger.info(f"pyproject.toml: setting [{section_name}]->{key}")

    def adjust_ruff(self):
        section_name = "tool.ruff"
        self._suggest(section_name, "target-version", "py310")

        section_name = "tool.ruff.lint"
        self._suggest(section_name, "select", ["E4", "E7", "E9", "F", "I", "UP"])

    def adjust_zestreleaser(self):
        section_name = "tool.zest-releaser"
        self._suggest(section_name, "release", False)

    def remove_old_sections(self):
        """Remove sections of old tools.

        For instance, isort had a `[tool.isort]` section. That's now obsoleted by ruff.
        """
        tool_super_section = self._contents.get("tool")
        if tool_super_section:
            if "isort" in tool_super_section:
                tool_super_section.remove("isort")
                logger.info("Removed [tool.isort] section")


if __name__ == "__main__":
    # Only called to write the documentation file.
    write_documentation()



import logging
import sys
from functools import cached_property
from pathlib import Path
from typing import Annotated

import jinja2
import typer

from nens_meta import nens_toml, pyproject_toml, utils

TEMPLATES_BASEDIR = Path(__file__).parent / "templates"

logger = logging.getLogger(__name__)


HEADER = """\
# Generated by nens-meta.
# See https://nens-meta.readthedocs.io/en/latest/config-files.html for info.
# If you want this file to be left alone, add "nens_meta_leave_alone" in
# all caps somewhere in this file in a comment.
#
"""


class TemplatedFile:
    project_dir: Path
    our_config: nens_toml.OurConfig
    template_name: str
    target_name: str  # Note: can be "subdir/some-file.txt"
    section_name: str
    only_create_dont_change: bool = False

    def __init__(self, project_dir: Path, our_config: nens_toml.OurConfig) -> None:
        self.project_dir = project_dir
        self.our_config = our_config

    @property
    def target(self) -> Path:
        return self.project_dir / self.target_name

    @property
    def environment(self) -> jinja2.Environment:
        return jinja2.Environment(
            loader=jinja2.FileSystemLoader(
                # pass one or more dirs! Handy for our purpose!
                [TEMPLATES_BASEDIR / "default"]
            ),
            keep_trailing_newline=True,
            trim_blocks=True,
            lstrip_blocks=True,
        )

    @property
    def template(self) -> jinja2.Template:
        return self.environment.get_template(self.template_name)

    def extra_options(self) -> dict:
        """Overwrite in subclasses to return some extra calculated options"""
        return {}

    @cached_property
    def our_options(self) -> dict:
        if not self.our_config.has_section_for(self.section_name):
            return {}
        return self.our_config.section_options(self.section_name)

    @cached_property
    def meta_options(self) -> dict:
        return self.our_config.section_options("meta")

    @cached_property
    def options(self) -> dict:
        result = {}
        result.update(self.meta_options)
        result.update(self.our_options)
        result.update(self.extra_options())
        return result

    @property
    def header(self) -> str:
        return HEADER.format(
            section_name=self.section_name, meta_version=self.options["meta_version"]
        )

    @cached_property
    def content(self) -> str:
        rendered = self.template.render(
            header=self.header,
            **self.options,
        )
        return utils.strip_whitespace(rendered)

    def create_dirs_if_needed(self):
        *directories, _ = self.target_name.split("/")
        if directories:
            target_dir = self.project_dir / "/".join(directories)
            if target_dir.exists():
                return
            target_dir.mkdir(parents=True)
            logger.info(f"Created directory {target_dir}")

    def write(self):
        """Copy the source template to the target, doing the jinja2 stuff"""
        self.create_dirs_if_needed()
        handle_extra_lines = True  # default
        if self.only_create_dont_change:
            handle_extra_lines = (
                False  # No need for this if we're not updating the file.
            )
            if self.target.exists():
                logger.debug(f"{self.target} already exists, skipping")
                return
        utils.write_if_changed(
            self.target, self.content, handle_extra_lines=handle_extra_lines
        )


class Editorconfig(TemplatedFile):
    """Wrapper around a project's editorconfig"""

    template_name = "editorconfig.j2"
    target_name = ".editorconfig"
    section_name = "editorconfig"


class Gitignore(TemplatedFile):
    """Wrapper around a project's gitignore"""

    template_name = "gitignore.j2"
    target_name = ".gitignore"
    section_name = "gitignore"


class Precommitconfig(TemplatedFile):
    """Wrapper around a project's .pre-commit-config.yaml"""

    template_name = "pre-commit-config.yaml.j2"
    target_name = ".pre-commit-config.yaml"
    section_name = "pre-commit-config"


class DependabotYml(TemplatedFile):
    """Wrapper around a dependabot.yml file"""

    template_name = "dependabot.yml.j2"
    target_name = ".github/dependabot.yml"
    section_name = "dependabot"


class MetaWorkflowYml(TemplatedFile):
    """Wrapper around a nens-meta.yml file"""

    template_name = "meta_workflow.yml.j2"
    target_name = ".github/workflows/nens-meta.yml"
    section_name = "meta_workflow"


class RequirementsYml(TemplatedFile):
    """Wrapper around an ansible requirements.yml file"""

    template_name = "requirements.yml.j2"
    target_name = "requirements.yml"
    section_name = "ansible"
    only_create_dont_change = True


def check_prerequisites(project_dir: Path):
    """Check prerequisites, exit if not met"""
    if not (project_dir / ".git").exists():
        if project_dir.absolute().name.startswith("{{ cookiecutter"):
            logger.info("Cookiecutter project dir detected")
        else:
            # No git and not the cookiecutter special case.
            logger.error("Project has no .git dir")
            sys.exit(1)
    if not nens_toml.nens_toml_file(project_dir).exists():
        nens_toml.create_if_missing(project_dir)
        logger.warning("No .nens.toml found, created one. Re-run after checking.")
        sys.exit(1)


def do_some_python_checks(project_dir: Path):
    """Run some checks to help identify issues and things you still need to do"""
    for file_to_check in project_dir.glob("*.outdated"):
        logger.warning(
            f"Check the old {file_to_check}: move settings to pyproject.toml, perhaps?"
        )
    website = "https://nens-meta.readthedocs.io"
    readme = Path("README.md")
    if readme.exists():
        if website not in readme.read_text():
            logger.warning(
                f"{website} is not mentioned in the readme as an instruction"
            )


def update_project(
    verbose: Annotated[bool, typer.Option(help="Verbose logging")] = False,
):  # pragma: no cover
    project_dir = Path(".")
    log_level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(level=log_level, format="%(levelname)-7s: %(message)s")
    check_prerequisites(project_dir)
    our_config = nens_toml.OurConfig(project_dir)
    our_config.write()

    if our_config.section_options("meta")["uses_python"]:
        if not pyproject_toml.pyproject_toml_file(project_dir).exists():
            pyproject_toml.create_if_missing(project_dir)
        options_for_project_config = {}
        options_for_project_config.update(our_config.section_options("meta"))
        options_for_project_config.update(our_config.section_options("pyprojecttoml"))
        project_config = pyproject_toml.PyprojectToml(
            project_dir, options_for_project_config
        )
        project_config.update()
        project_config.write()

    # Grab editorconfig table and pass it along. Or rather the whole thing?
    editorconfig = Editorconfig(project_dir, our_config)
    editorconfig.write()
    gitignore = Gitignore(project_dir, our_config)
    gitignore.write()
    precommitconfig = Precommitconfig(project_dir, our_config)
    precommitconfig.write()
    dependabot_yml = DependabotYml(project_dir, our_config)
    dependabot_yml.write()
    meta_workflow_yml = MetaWorkflowYml(project_dir, our_config)
    meta_workflow_yml.write()

    if our_config.section_options("meta")["uses_ansible"]:
        requirements_yml = RequirementsYml(project_dir, our_config)
        requirements_yml.write()

    if our_config.section_options("meta")["uses_python"]:
        do_some_python_checks(project_dir)


def main():  # pragma: no cover
    typer.run(update_project)



from pathlib import Path

from pytest_mock.plugin import MockerFixture

from nens_meta import utils


def test_strip_whitespace():
    content = "Example\n    \ncontent  \nend\n\n\n"
    expected = "Example\n\ncontent\nend\n"
    assert utils.strip_whitespace(content) == expected


def test_extract_extra_lines1():
    # Empty content should be ok
    assert utils._extract_extra_lines("") == ""


def test_extract_extra_lines2():
    content = f"bla\n{utils.EXTRA_LINES_MARKER}reinout\n"
    assert utils._extract_extra_lines(content) == "reinout\n"


def test_write_if_changed1(tmp_path: Path):
    # Just write something to a new file. The extra lines marker will also be in there.
    f = tmp_path / "sample.txt"
    utils.write_if_changed(f, "test")
    assert f.exists()
    content = f.read_text()
    assert content.startswith("test")
    assert utils.EXTRA_LINES_MARKER in content


def test_write_if_changed2(tmp_path: Path):
    # Write something new to an existing file.
    f = tmp_path / "sample.txt"
    f.write_text("bla bla")
    utils.write_if_changed(f, "test")
    content = f.read_text()
    assert content.startswith("test")


def test_write_if_changed3(tmp_path: Path, mocker: MockerFixture):
    # Don't change an existing file if it is not needed.
    f = tmp_path / "sample.txt"
    f.write_text("test\n\n" + utils.EXTRA_LINES_MARKER)
    writer = mocker.spy(Path, "write_text")
    utils.write_if_changed(f, "test\n")
    writer.assert_not_called()


def test_write_if_changed4(tmp_path: Path):
    # Don't write something if the file should be left alone
    f = tmp_path / "sample.txt"
    f.write_text("bla bla\n# NENS_META_LEAVE_ALONE\n")
    utils.write_if_changed(f, "test")
    content = f.read_text()
    assert content.startswith("bla bla")
    assert (tmp_path / "sample.txt.suggestion").exists()


def test_uses_python1():
    # We ourselves are a python project.
    ourselves = Path(__file__).parent.parent.parent
    assert utils.uses_python(ourselves)


def test_uses_python2(tmp_path: Path):
    # An empty dir is not a python project
    assert not utils.uses_python(tmp_path)


def test_uses_ansible1():
    # We ourselves have no ansible/ dir.
    ourselves = Path(__file__).parent.parent.parent
    assert not utils.uses_ansible(ourselves)


def test_uses_ansible2(tmp_path: Path):
    (tmp_path / "ansible").mkdir()
    assert utils.uses_ansible(tmp_path)



from pathlib import Path

import pytest

from nens_meta import pyproject_toml


@pytest.fixture
def empty_python_config(tmp_path: Path) -> pyproject_toml.PyprojectToml:
    pyproject_toml.create_if_missing(tmp_path)
    return pyproject_toml.PyprojectToml(tmp_path, {})


def test_pyproject_toml_file(tmp_path: Path):
    assert pyproject_toml.pyproject_toml_file(tmp_path).name == "pyproject.toml"


def test_create_if_missing(tmp_path: Path):
    pyproject_toml.create_if_missing(tmp_path)
    assert (tmp_path / "pyproject.toml").exists()


def test_read(empty_python_config: pyproject_toml.PyprojectToml):
    assert empty_python_config._contents == {}


def test_get_or_create_section1(empty_python_config: pyproject_toml.PyprojectToml):
    empty_python_config.get_or_create_section("reinout")
    empty_python_config.write()
    assert "[reinout]" in empty_python_config._config_file.read_text()


def test_get_or_create_section(empty_python_config: pyproject_toml.PyprojectToml):
    empty_python_config.get_or_create_section("reinout.van")
    empty_python_config.write()
    assert "[reinout.van]" in empty_python_config._config_file.read_text()


def test_adjust_ruff(empty_python_config: pyproject_toml.PyprojectToml):
    empty_python_config.adjust_ruff()
    empty_python_config.write()
    assert "[tool.ruff.lint]" in empty_python_config._config_file.read_text()
    assert "target-version" in empty_python_config._config_file.read_text()


def test_adjust_zestreleaser(empty_python_config: pyproject_toml.PyprojectToml):
    empty_python_config.adjust_zestreleaser()
    empty_python_config.write()
    assert "release = false" in empty_python_config._config_file.read_text()


def test_remove_old_sections(empty_python_config: pyproject_toml.PyprojectToml):
    empty_python_config._config_file.write_text("[tool.isort]\nreinout = 1972")
    empty_python_config.read()
    empty_python_config.remove_old_sections()
    empty_python_config.write()
    assert "reinout" not in empty_python_config._config_file.read_text()


def test_write_documentation():
    pyproject_toml.write_documentation()
